;; Generated package description from rcirc-color.el  -*- no-byte-compile: t -*-
(define-package "rcirc-color" "0.4.4.0.20221012.112425" "color nicks" '((emacs "24.4")) :commit "6881c924c055e722dd563468c9d48cd1955a6853" :url "https://elpa.gnu.org/packages/rcirc-color.html" :authors '(("Alex Schroeder" . "alex@gnu.org")) :maintainer '("Alex Schroeder" . "alex@gnu.org") :keywords '("comm"))
